SystemOrganization addCategory: #'Collections-Stack'!

Link subclass: #StackLink
	instanceVariableNames: 'element'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Stack'!

!StackLink commentStamp: '<historical>' prior: 0!
I implement an element of a stack. I'm a container for any type of object, saved into the 'element' variable. My superclass Link allows me to be part of a LinkedList.!

!StackLink classSide methodsFor: 'instance creation' stamp: 'dc 7/25/2005 10:15'!
with: anObject 
	^ self new element: anObject! !

!StackLink methodsFor: 'accessing' stamp: 'dc 7/24/2005 15:34'!
element
	^element! !

!StackLink methodsFor: 'accessing' stamp: 'dc 7/25/2005 10:16'!
element: anObject 
	"Any kind of Object."
	element := anObject! !

!StackLink methodsFor: 'printing' stamp: 'dc 7/25/2005 10:15'!
printOn: aStream 
	aStream nextPutAll: self class printString;
		 nextPutAll: ' with: ';
		 nextPutAll: self element printString! !

Object subclass: #Stack
	instanceVariableNames: 'linkedList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Stack'!

!Stack commentStamp: 'dc 7/24/2005 15:41' prior: 0!
I implement a simple Stack. #push: adds a new object of any kind on top of the stack. #pop returns the first element and remove it from the stack. #top answer the first element of the stack without removing it.!

!Stack methodsFor: 'private' stamp: 'dc 7/25/2005 10:05'!
errorEmptyStack
	self error: 'this stack is empty'! !

!Stack methodsFor: 'testing' stamp: 'stephane.ducasse 12/5/2008 18:14'!
ifEmpty: emptyBlock ifNotEmpty: notEmptyBlock
	"Evaluate emptyBlock if I'm empty, notEmptyBlock otherwise"
	" If the notEmptyBlock has an argument, eval with the receiver as its argument"
	"copied from Collection of course"

	^ self isEmpty ifTrue: emptyBlock ifFalse: [notEmptyBlock valueWithPossibleArgument: self]! !

!Stack methodsFor: 'testing' stamp: 'stephane.ducasse 12/5/2008 18:21'!
ifEmpty: emptyBlock ifNotEmptyDo: notEmptyBlock
	"Evaluate emptyBlock if I'm empty, notEmptyBlock otherwise"
	"Evaluate the notEmptyBlock with the receiver as its argument"
	"copied from Collection"
	
	^ self isEmpty ifTrue: emptyBlock ifFalse: [notEmptyBlock value: self]! !

!Stack methodsFor: 'testing' stamp: 'stephane.ducasse 12/5/2008 18:15'!
ifNotEmpty: aBlock
	"Evaluate the given block unless the receiver is empty.

      If the block has an argument, eval with the receiver as its argument,
      but it might be better to use ifNotEmptyDo: to make the code easier to
      understand"
	"copied from Collection"
	^self isEmpty ifFalse: [aBlock valueWithPossibleArgument: self].
! !

!Stack methodsFor: 'testing' stamp: 'stephane.ducasse 12/5/2008 18:16'!
ifNotEmptyDo: aBlock
	"Evaluate the given block with the receiver as its argument."
	"copied from Collection"
	^self isEmpty ifFalse: [aBlock value: self].
! !

!Stack methodsFor: 'testing' stamp: 'stephane.ducasse 12/5/2008 18:22'!
ifNotEmptyDo: notEmptyBlock ifEmpty: emptyBlock
	"Evaluate emptyBlock if I'm empty, notEmptyBlock otherwise
	Evaluate the notEmptyBlock with the receiver as its argument"
	"copied from Collection"

	^ self isEmpty ifFalse: [notEmptyBlock value: self] ifTrue: emptyBlock! !

!Stack methodsFor: 'initialization' stamp: 'dc 7/25/2005 11:39'!
initialize
	super initialize.
	linkedList := LinkedList new! !

!Stack methodsFor: 'testing' stamp: 'dc 7/25/2005 10:05'!
isEmpty
	^ self linkedList isEmpty! !

!Stack methodsFor: 'testing' stamp: 'stephane.ducasse 12/5/2008 18:16'!
isEmptyOrNil
	"Answer whether the receiver contains any elements, or is nil.  Useful in numerous situations where one wishes the same reaction to an empty collection or to nil"
	"copied from Collection"
	
	^ self isEmpty! !

!Stack methodsFor: 'private' stamp: 'dc 7/25/2005 10:20'!
linkedList
	"The stack is implemented with a LinkedList. Do NOT call this function, it  
	is for private use !!"
	^ linkedList! !

!Stack methodsFor: 'private' stamp: 'dc 7/25/2005 10:05'!
notEmptyCheck
	"Ensure the stack is not empty."
	self isEmpty
		ifTrue: [self errorEmptyStack]! !

!Stack methodsFor: 'removing' stamp: 'dc 7/24/2005 16:16'!
pop
	"Returns the first element and remove it from the stack."

	self notEmptyCheck.
	^self linkedList removeFirst element! !

!Stack methodsFor: 'adding' stamp: 'dc 7/25/2005 10:22'!
push: anObject 
	"Adds a new object of any kind on top of the stack."
	self linkedList
		addFirst: (StackLink with: anObject).
	^ anObject.! !

!Stack methodsFor: 'accessing' stamp: 'dc 7/25/2005 10:04'!
size
	"How many objects in me ?"
	^ self linkedList size! !

!Stack methodsFor: 'accessing' stamp: 'sd 3/25/2006 15:05'!
top
	"Answer the first element of the stack without removing it."
	self notEmptyCheck.
	^ self linkedList first element! !
